@@ -21,7 +21,6 @@
 #include <KCModuleInfo>
 #include <KCMUtils/KCModuleLoader>
 #include <KConfigGroup>
-#include <KDeclarative/KDeclarative>
 #include <KDescendantsProxyModel>
 #include <KLocalizedContext>
 #include <KLocalizedString>
@@ -204,17 +203,9 @@ public:
         case Qt::UserRole:
             return QVariant::fromValue(mi);
         case Qt::DisplayRole:
-            if (mi->service() && mi->service()->isValid()) {
-                return mi->service()->name();
-            } else {
-                return QVariant();
-            }
+            return mi->name();
         case Qt::DecorationRole:
-            if (mi->service() && mi->service()->isValid()) {
-                return mi->service()->icon();
-            } else {
-                return QVariant();
-            }
+            return mi->iconName();
         case ResultModel::ScoreRole:
             return QSortFilterProxyModel::data(index, ResultModel::ScoreRole).toInt();
         default:
@@ -256,7 +247,6 @@ public:
     FocusHackWidget *mainWidget = nullptr;
     QQuickWidget *placeHolderWidget = nullptr;
     QHBoxLayout *mainLayout = nullptr;
-    KDeclarative::KDeclarative kdeclarative;
     MenuModel *model = nullptr;
     MenuProxyModel *categorizedModel = nullptr;
     MenuProxyModel *searchModel = nullptr;
@@ -601,18 +591,25 @@ void SidebarMode::moduleLoaded()
 
 void SidebarMode::updateDefaults()
 {
-    // When the landing page is loaded, we don't have activeCategoryRow
+    // When the landing page is loaded, we don't have activeCategoryRow and need to reload everything
     if (d->activeCategoryRow < 0) {
+        refreshDefaults();
         return;
     }
     QModelIndex categoryIdx = d->categorizedModel->index(d->activeCategoryRow, 0);
     auto item = categoryIdx.data(Qt::UserRole).value<MenuItem *>();
     Q_ASSERT(item);
+
     // If subcategory exist update from subcategory, unless this category is owned by a kcm
     if (!item->children().isEmpty() && d->activeSubCategoryRow > -1) {
         auto subCateogryIdx = d->subCategoryModel->index(d->activeSubCategoryRow, 0);
         item = subCateogryIdx.data(Qt::UserRole).value<MenuItem *>();
     }
+    // In case we are updating a parent like LnF we refresh all defaults
+    if (item->isCategoryOwner() && item->parent() != rootItem()) {
+        refreshDefaults();
+        return;
+    }
 
     auto *moduleData = KCModuleLoader::loadModuleData(item->item());
     if (moduleData) {
@@ -701,11 +698,8 @@ qreal SidebarMode::headerHeight() const
     return d->moduleView->headerHeight();
 }
 
-void SidebarMode::toggleDefaultsIndicatorsVisibility()
+void SidebarMode::refreshDefaults()
 {
-    d->m_defaultsIndicatorsVisible = !d->m_defaultsIndicatorsVisible;
-    d->moduleView->moduleShowDefaultsIndicators(d->m_defaultsIndicatorsVisible);
-
     if (d->m_defaultsIndicatorsVisible) {
         for (int i = 0; i < d->flatModel->rowCount(); ++i) {
             QModelIndex idx = d->flatModel->index(i, 0);
@@ -724,6 +718,12 @@ void SidebarMode::toggleDefaultsIndicatorsVisibility()
             }
         }
     }
+}
+void SidebarMode::toggleDefaultsIndicatorsVisibility()
+{
+    d->m_defaultsIndicatorsVisible = !d->m_defaultsIndicatorsVisible;
+    d->moduleView->moduleShowDefaultsIndicators(d->m_defaultsIndicatorsVisible);
+    refreshDefaults();
     config().writeEntry("HighlightNonDefaultSettings", d->m_defaultsIndicatorsVisible);
     Q_EMIT defaultsIndicatorsVisibleChanged();
 }
@@ -790,9 +790,7 @@ void SidebarMode::initWidget()
     d->package = KPackage::PackageLoader::self()->loadPackage(QStringLiteral("KPackage/GenericQML"));
     d->package.setPath(QStringLiteral("org.kde.systemsettings.sidebar"));
 
-    d->kdeclarative.setDeclarativeEngine(d->quickWidget->engine());
-    d->kdeclarative.setupEngine(d->quickWidget->engine());
-    d->kdeclarative.setupContext();
+    d->quickWidget->engine()->rootContext()->setContextObject(new KLocalizedContext(d->quickWidget));
 
     d->quickWidget->setSource(QUrl::fromLocalFile(d->package.filePath("mainscript")));
 
